@@ -10,11 +10,11 @@
   log = window.calibre_utils.log;
 
   runscripts = function(parent) {
-    var i, len, ref, results, script;
+    var j, len, ref, results, script;
     ref = parent.getElementsByTagName('script');
     results = [];
-    for (i = 0, len = ref.length; i < len; i++) {
-      script = ref[i];
+    for (j = 0, len = ref.length; j < len; j++) {
+      script = ref[j];
       results.push(eval(script.text || script.textContent || script.innerHTML || ''));
     }
     return results;
@@ -35,10 +35,10 @@
   };
 
   has_start_text = function(elem) {
-    var c, i, len, ref, ref1;
+    var c, j, len, ref, ref1;
     ref = elem.childNodes;
-    for (i = 0, len = ref.length; i < len; i++) {
-      c = ref[i];
+    for (j = 0, len = ref.length; j < len; j++) {
+      c = ref[j];
       if ((ref1 = c.nodeType) !== Node.TEXT_NODE && ref1 !== Node.COMMENT_NODE && ref1 !== Node.PROCESSING_INSTRUCTION_NODE) {
         break;
       }
@@ -89,7 +89,7 @@
     }
 
     PagedDisplay.prototype.read_document_margins = function() {
-      var i, j, k, len, len1, len2, prop, pxval, ref, ref1, ref2, rule, sheet, tmp, val;
+      var j, k, len, len1, len2, m, prop, pxval, ref, ref1, ref2, rule, sheet, tmp, val;
       if (this.document_margins === null) {
         this.document_margins = {
           left: null,
@@ -102,16 +102,16 @@
         tmp.style.position = 'absolute';
         document.body.appendChild(tmp);
         ref = document.styleSheets;
-        for (i = 0, len = ref.length; i < len; i++) {
-          sheet = ref[i];
+        for (j = 0, len = ref.length; j < len; j++) {
+          sheet = ref[j];
           if (sheet.rules) {
             ref1 = sheet.rules;
-            for (j = 0, len1 = ref1.length; j < len1; j++) {
-              rule = ref1[j];
+            for (k = 0, len1 = ref1.length; k < len1; k++) {
+              rule = ref1[k];
               if (rule.type === CSSRule.PAGE_RULE) {
                 ref2 = ['left', 'top', 'bottom', 'right'];
-                for (k = 0, len2 = ref2.length; k < len2; k++) {
-                  prop = ref2[k];
+                for (m = 0, len2 = ref2.length; m < len2; m++) {
+                  prop = ref2[m];
                   val = rule.style.getPropertyValue('margin-' + prop);
                   if (val) {
                     tmp.style.height = val;
@@ -177,11 +177,11 @@
     };
 
     PagedDisplay.prototype.handle_rtl_body = function(body_style) {
-      var i, len, node, ref;
+      var j, len, node, ref;
       if (body_style.direction === "rtl") {
         ref = document.body.childNodes;
-        for (i = 0, len = ref.length; i < len; i++) {
-          node = ref[i];
+        for (j = 0, len = ref.length; j < len; j++) {
+          node = ref[j];
           if (node.nodeType === node.ELEMENT_NODE && window.getComputedStyle(node).direction === "rtl") {
             node.style.setProperty("direction", "rtl");
           }
@@ -192,7 +192,7 @@
     };
 
     PagedDisplay.prototype.layout = function(is_single_page) {
-      var adjust, body_style, body_width, bs, c, c2, col_width, cprop, edge, eh, elems, fgcolor, first_layout, has_svg, has_viewport, i, j, k, len, len1, len2, len3, m, n, ncols, num, only_img, priority, prop, ref, ref1, ref2, ref3, ref4, rule, sheet, single_screen, sm, val, ww;
+      var adjust, body_style, body_width, bs, c, c2, col_width, cprop, edge, eh, elems, fgcolor, first_layout, has_svg, has_viewport, j, k, len, len1, len2, len3, m, n, ncols, num, o, only_img, priority, prop, ref, ref1, ref2, ref3, ref4, rule, sheet, single_screen, sm, val, ww;
       if (is_single_page == null) {
         is_single_page = false;
       }
@@ -241,6 +241,7 @@
       bs.setProperty('-webkit-column-gap', 2 * sm + 'px');
       bs.setProperty('-webkit-column-width', col_width + 'px');
       bs.setProperty('-webkit-column-rule', '0px inset blue');
+      bs.setProperty('column-fill', 'auto');
       bs.setProperty('-webkit-margin-collapse', 'separate');
       c = first_child(document.body);
       if (c !== null) {
@@ -269,8 +270,8 @@
       bs.setProperty('padding-left', sm + 'px');
       bs.setProperty('padding-right', sm + 'px');
       ref1 = ['left', 'right', 'top', 'bottom'];
-      for (i = 0, len = ref1.length; i < len; i++) {
-        edge = ref1[i];
+      for (j = 0, len = ref1.length; j < len; j++) {
+        edge = ref1[j];
         bs.setProperty('margin-' + edge, '0px');
         bs.setProperty('border-' + edge + '-width', '0px');
       }
@@ -279,16 +280,16 @@
       bs.setProperty('min-height', '0');
       bs.setProperty('max-height', 'none');
       ref2 = document.styleSheets;
-      for (j = 0, len1 = ref2.length; j < len1; j++) {
-        sheet = ref2[j];
+      for (k = 0, len1 = ref2.length; k < len1; k++) {
+        sheet = ref2[k];
         if (sheet.rules) {
           ref3 = sheet.rules;
-          for (k = 0, len2 = ref3.length; k < len2; k++) {
-            rule = ref3[k];
+          for (m = 0, len2 = ref3.length; m < len2; m++) {
+            rule = ref3[m];
             if (rule.type === CSSRule.STYLE_RULE) {
               ref4 = ['page-break-before', 'page-break-after', 'page-break-inside'];
-              for (m = 0, len3 = ref4.length; m < len3; m++) {
-                prop = ref4[m];
+              for (o = 0, len3 = ref4.length; o < len3; o++) {
+                prop = ref4[o];
                 val = rule.style.getPropertyValue(prop);
                 if (val) {
                   cprop = '-webkit-column-' + prop.substr(5);
@@ -355,7 +356,7 @@
     };
 
     PagedDisplay.prototype.update_header_footer = function(pagenum) {
-      var author, cls, has_images, i, img, j, len, len1, ref, ref1, section, title, tl_section;
+      var author, cls, has_images, img, j, k, len, len1, ref, ref1, section, title, tl_section;
       has_images = false;
       this.header_footer_images = [];
       if (this.hf_style !== null) {
@@ -374,8 +375,8 @@
         this.header.innerHTML = this.header_template.replace(/_PAGENUM_/g, pagenum + "").replace(/_TITLE_/g, title + "").replace(/_AUTHOR_/g, author + "").replace(/_TOP_LEVEL_SECTION_/g, tl_section + "").replace(/_SECTION_/g, section + "");
         runscripts(this.header);
         ref = this.header.getElementsByTagName('img');
-        for (i = 0, len = ref.length; i < len; i++) {
-          img = ref[i];
+        for (j = 0, len = ref.length; j < len; j++) {
+          img = ref[j];
           this.header_footer_images.push(img);
           has_images = true;
         }
@@ -384,8 +385,8 @@
         this.footer.innerHTML = this.footer_template.replace(/_PAGENUM_/g, pagenum + "").replace(/_TITLE_/g, title + "").replace(/_AUTHOR_/g, author + "").replace(/_TOP_LEVEL_SECTION_/g, tl_section + "").replace(/_SECTION_/g, section + "");
         runscripts(this.footer);
         ref1 = this.header.getElementsByTagName('img');
-        for (j = 0, len1 = ref1.length; j < len1; j++) {
-          img = ref1[j];
+        for (k = 0, len1 = ref1.length; k < len1; k++) {
+          img = ref1[k];
           this.header_footer_images.push(img);
           has_images = true;
         }
@@ -394,10 +395,10 @@
     };
 
     PagedDisplay.prototype.header_footer_images_loaded = function() {
-      var i, img, len, ref;
+      var img, j, len, ref;
       ref = this.header_footer_images;
-      for (i = 0, len = ref.length; i < len; i++) {
-        img = ref[i];
+      for (j = 0, len = ref.length; j < len; j++) {
+        img = ref[j];
         if (!img.complete) {
           return false;
         }
@@ -407,16 +408,21 @@
     };
 
     PagedDisplay.prototype.fit_images = function() {
-      var br, col, col_width, data, doc, i, images, img, j, k, left, len, len1, len2, max_width, maxh, previously_limited, ref, ref1, results, rleft, rright, vimages, width;
+      var bounding_rects, br, col, col_width, data, doc, i, images, img, img_tags, j, k, left, len, len1, len2, m, max_width, maxh, o, previously_limited, ref, ref1, results, rleft, rright, vimages, width;
       images = [];
       vimages = [];
+      bounding_rects = [];
+      img_tags = document.getElementsByTagName('img');
+      for (j = 0, len = img_tags.length; j < len; j++) {
+        img = img_tags[j];
+        bounding_rects.push(img.getBoundingClientRect());
+      }
       maxh = this.current_page_height;
-      ref = document.getElementsByTagName('img');
-      for (i = 0, len = ref.length; i < len; i++) {
-        img = ref[i];
+      for (i = k = 0, ref = img_tags.length; 0 <= ref ? k < ref : k > ref; i = 0 <= ref ? ++k : --k) {
+        img = img_tags[i];
         previously_limited = calibre_utils.retrieve(img, 'width-limited', false);
         data = calibre_utils.retrieve(img, 'img-data', null);
-        br = img.getBoundingClientRect();
+        br = bounding_rects[i];
         if (data === null) {
           data = {
             'left': br.left,
@@ -445,14 +451,14 @@
         }
         img.style.setProperty('-webkit-column-break-inside', 'avoid');
       }
-      for (j = 0, len1 = images.length; j < len1; j++) {
-        ref1 = images[j], img = ref1[0], max_width = ref1[1];
+      for (m = 0, len1 = images.length; m < len1; m++) {
+        ref1 = images[m], img = ref1[0], max_width = ref1[1];
         img.style.setProperty('max-width', max_width + 'px');
         calibre_utils.store(img, 'width-limited', true);
       }
       results = [];
-      for (k = 0, len2 = vimages.length; k < len2; k++) {
-        img = vimages[k];
+      for (o = 0, len2 = vimages.length; o < len2; o++) {
+        img = vimages[o];
         data = calibre_utils.retrieve(img, 'img-data', null);
         img.style.setProperty('-webkit-column-break-before', 'always');
         img.style.setProperty('max-height', maxh + 'px');
@@ -723,7 +729,7 @@
     };
 
     PagedDisplay.prototype.current_cfi = function() {
-      var ans, c, cfi, curx, cury, deltax, deltay, err, i, left, len, ref, ref1, ref2, ref3, right, x;
+      var ans, c, cfi, curx, cury, deltax, deltay, err, j, left, len, ref, ref1, ref2, ref3, right, x;
       ans = null;
       if ((window.cfi == null) || (((ref = window.mathjax) != null ? ref.math_present : void 0) && !((ref1 = window.mathjax) != null ? ref1.math_loaded : void 0))) {
         return ans;
@@ -731,8 +737,8 @@
       if (this.in_paged_mode) {
         c = this.current_column_location();
         ref2 = [c, c - this.page_width, c + this.page_width];
-        for (i = 0, len = ref2.length; i < len; i++) {
-          x = ref2[i];
+        for (j = 0, len = ref2.length; j < len; j++) {
+          x = ref2[j];
           ref3 = [x, x + this.page_width], left = ref3[0], right = ref3[1];
           if (left < 0 || right > document.body.scrollWidth) {
             continue;
