@@ -1,4 +1,4 @@
-from __future__ import with_statement
+from __future__ import absolute_import, division, print_function, unicode_literals
 __license__   = 'GPL v3'
 __copyright__ = '2008, Kovid Goyal <kovid at kovidgoyal.net>'
 
@@ -6,6 +6,7 @@ import os, sys, zipfile, importlib
 
 from calibre.constants import numeric_version, iswindows, isosx
 from calibre.ptempfile import PersistentTemporaryFile
+from polyglot.builtins import unicode_type
 
 platform = 'linux'
 if iswindows:
@@ -26,7 +27,7 @@ class Plugin(object):  # {{{
     '''
     A calibre plugin. Useful members include:
 
-       * ``self.plugin_path``: Stores path to the zip file that contains
+       * ``self.plugin_path``: Stores path to the ZIP file that contains
                                this plugin or None if it is a builtin
                                plugin
        * ``self.site_customization``: Stores a customization string entered
@@ -62,9 +63,9 @@ class Plugin(object):  # {{{
     author         = _('Unknown')
 
     #: When more than one plugin exists for a filetype,
-    #: the plugins are run in order of decreasing priority
-    #: i.e. plugins with higher priority will be run first.
-    #: The highest possible priority is ``sys.maxint``.
+    #: the plugins are run in order of decreasing priority.
+    #: Plugins with higher priority will be run first.
+    #: The highest possible priority is ``sys.maxsize``.
     #: Default priority is 1.
     priority = 1
 
@@ -91,7 +92,7 @@ class Plugin(object):  # {{{
         plugin will be initialized for every new worker process.
 
         Perform any plugin specific initialization here, such as extracting
-        resources from the plugin zip file. The path to the zip file is
+        resources from the plugin ZIP file. The path to the ZIP file is
         available as ``self.plugin_path``.
 
         Note that ``self.site_customization`` is **not** available at this point.
@@ -194,7 +195,7 @@ class Plugin(object):  # {{{
             config_dialog.exec_()
 
             if config_dialog.result() == QDialog.Accepted:
-                sc = unicode(sc.text()).strip()
+                sc = unicode_type(sc.text()).strip()
                 customize_plugin(self, sc)
 
         geom = bytearray(config_dialog.saveGeometry())
@@ -210,13 +211,13 @@ class Plugin(object):  # {{{
         For example to load an image::
 
             pixmap = QPixmap()
-            pixmap.loadFromData(self.load_resources(['images/icon.png']).itervalues().next())
+            pixmap.loadFromData(self.load_resources(['images/icon.png'])['images/icon.png'])
             icon = QIcon(pixmap)
 
-        :param names: List of paths to resources in the zip file using / as separator
+        :param names: List of paths to resources in the ZIP file using / as separator
 
         :return: A dictionary of the form ``{name: file_contents}``. Any names
-                 that were not found in the zip file will not be present in the
+                 that were not found in the ZIP file will not be present in the
                  dictionary.
 
         '''
@@ -246,7 +247,7 @@ class Plugin(object):  # {{{
         :param gui: If True return HTML help, otherwise return plain text help.
 
         '''
-        raise NotImplementedError
+        raise NotImplementedError()
 
     def temporary_file(self, suffix):
         '''
@@ -276,8 +277,8 @@ class Plugin(object):  # {{{
         if self.plugin_path is not None:
             from calibre.utils.zipfile import ZipFile
             zf = ZipFile(self.plugin_path)
-            extensions = set([x.rpartition('.')[-1].lower() for x in
-                zf.namelist()])
+            extensions = {x.rpartition('.')[-1].lower() for x in
+                zf.namelist()}
             zip_safe = True
             for ext in ('pyd', 'so', 'dll', 'dylib'):
                 if ext in extensions:
@@ -402,7 +403,7 @@ class MetadataReaderPlugin(Plugin):  # {{{
     '''
     A plugin that implements reading metadata from a set of file types.
     '''
-    #: Set of file types for which this plugin should be run
+    #: Set of file types for which this plugin should be run.
     #: For example: ``set(['lit', 'mobi', 'prc'])``
     file_types     = set([])
 
@@ -434,7 +435,7 @@ class MetadataWriterPlugin(Plugin):  # {{{
     '''
     A plugin that implements reading metadata from a set of file types.
     '''
-    #: Set of file types for which this plugin should be run
+    #: Set of file types for which this plugin should be run.
     #: For example: ``set(['lit', 'mobi', 'prc'])``
     file_types     = set([])
 
@@ -470,22 +471,20 @@ class CatalogPlugin(Plugin):  # {{{
 
     resources_path = None
 
-    #: Output file type for which this plugin should be run
+    #: Output file type for which this plugin should be run.
     #: For example: 'epub' or 'xml'
     file_types = set([])
 
     type = _('Catalog generator')
 
-    #: CLI parser options specific to this plugin, declared as namedtuple Option::
+    #: CLI parser options specific to this plugin, declared as namedtuple Option:
+    #:
+    #:   from collections import namedtuple
+    #:   Option = namedtuple('Option', 'option, default, dest, help')
+    #:   cli_options = [Option('--catalog-title', default = 'My Catalog',
+    #:   dest = 'catalog_title', help = (_('Title of generated catalog. \nDefault:') + " '" + '%default' + "'"))]
+    #:   cli_options parsed in calibre.db.cli.cmd_catalog:option_parser()
     #:
-    #:  from collections import namedtuple
-    #:  Option = namedtuple('Option', 'option, default, dest, help')
-    #:  cli_options = [Option('--catalog-title',
-    #:                       default = 'My Catalog',
-    #:                       dest = 'catalog_title',
-    #:                       help = (_('Title of generated catalog. \nDefault:') + " '" +
-    #:                       '%default' + "'"))]
-    #:  cli_options parsed in library.cli:catalog_option_parser()
     cli_options = []
 
     def _field_sorter(self, key):
@@ -508,11 +507,10 @@ class CatalogPlugin(Plugin):  # {{{
 
     def get_output_fields(self, db, opts):
         # Return a list of requested fields
-        all_std_fields = set(
-                          ['author_sort','authors','comments','cover','formats',
+        all_std_fields = {'author_sort','authors','comments','cover','formats',
                            'id','isbn','library_name','ondevice','pubdate','publisher',
                            'rating','series_index','series','size','tags','timestamp',
-                           'title_sort','title','uuid','languages','identifiers'])
+                           'title_sort','title','uuid','languages','identifiers'}
         all_custom_fields = set(db.custom_field_keys())
         for field in list(all_custom_fields):
             fm = db.field_metadata[field]
@@ -546,7 +544,7 @@ class CatalogPlugin(Plugin):  # {{{
     def initialize(self):
         '''
         If plugin is not a built-in, copy the plugin's .ui and .py files from
-        the zip file to $TMPDIR.
+        the ZIP file to $TMPDIR.
         Tab will be dynamically generated and added to the Catalog Options dialog in
         calibre.gui2.dialogs.catalog.py:Catalog
         '''
@@ -565,7 +563,7 @@ class CatalogPlugin(Plugin):  # {{{
                 try:
                     resources.extract(file, self.resources_path)
                 except:
-                    print " customize:__init__.initialize(): %s not found in %s" % (file, os.path.basename(self.plugin_path))
+                    print(" customize:__init__.initialize(): %s not found in %s" % (file, os.path.basename(self.plugin_path)))
                     continue
             resources.close()
 
@@ -595,7 +593,7 @@ class InterfaceActionBase(Plugin):  # {{{
 
     supported_platforms = ['windows', 'osx', 'linux']
     author         = 'Kovid Goyal'
-    type = _('User Interface Action')
+    type = _('User interface action')
     can_be_disabled = False
 
     actual_plugin = None
@@ -734,7 +732,7 @@ class ViewerPlugin(Plugin):  # {{{
     type = _('Viewer')
 
     '''
-    These plugins are used to add functionality to the calibre viewer.
+    These plugins are used to add functionality to the calibre E-book viewer.
     '''
 
     def load_fonts(self):
@@ -745,7 +743,7 @@ class ViewerPlugin(Plugin):  # {{{
             def load_fonts():
                 from PyQt5.Qt import QFontDatabase
                 font_data = get_resources(['myfont1.ttf', 'myfont2.ttf'])
-                for raw in font_data.itervalues():
+                for raw in font_data.values():
                     QFontDatabase.addApplicationFontFromData(raw)
         '''
         pass
