@@ -1,3 +1,5 @@
+from __future__ import absolute_import, division, print_function, unicode_literals
+
 """ elements.py -- replacements and helpers for ElementTree """
 
 from polyglot.builtins import unicode_type, string_or_bytes
@@ -14,7 +16,7 @@ class ElementWriter(object):
         self.outputEncodingName = outputEncodingName
 
     def _encodeCdata(self, rawText):
-        if type(rawText) is str:
+        if isinstance(rawText, bytes):
             rawText = rawText.decode(self.sourceEncoding)
 
         text = rawText.replace("&", "&amp;")
@@ -23,20 +25,20 @@ class ElementWriter(object):
         return text
 
     def _writeAttribute(self, f, name, value):
-        f.write(u' %s="' % unicode_type(name))
+        f.write(' %s="' % unicode_type(name))
         if not isinstance(value, string_or_bytes):
             value = unicode_type(value)
         value = self._encodeCdata(value)
         value = value.replace('"', '&quot;')
         f.write(value)
-        f.write(u'"')
+        f.write('"')
 
     def _writeText(self, f, rawText):
         text = self._encodeCdata(rawText)
         f.write(text)
 
     def _write(self, f, e):
-        f.write(u'<' + unicode_type(e.tag))
+        f.write('<' + unicode_type(e.tag))
 
         attributes = e.items()
         attributes.sort()
@@ -44,7 +46,7 @@ class ElementWriter(object):
             self._writeAttribute(f, name, value)
 
         if e.text is not None or len(e) > 0:
-            f.write(u'>')
+            f.write('>')
 
             if e.text:
                 self._writeText(f, e.text)
@@ -52,11 +54,11 @@ class ElementWriter(object):
             for e2 in e:
                 self._write(f, e2)
 
-            f.write(u'</%s>' % e.tag)
+            f.write('</%s>' % e.tag)
         else:
             if self.spaceBeforeClose:
                 f.write(' ')
-            f.write(u'/>')
+            f.write('/>')
 
         if e.tail is not None:
             self._writeText(f, e.tail)
@@ -67,10 +69,10 @@ class ElementWriter(object):
         buffer = []
         x.write = buffer.append
         self.write(x)
-        return u''.join(buffer)
+        return ''.join(buffer)
 
     def write(self, f):
         if self.header:
-            f.write(u'<?xml version="1.0" encoding="%s"?>\n' % self.outputEncodingName)
+            f.write('<?xml version="1.0" encoding="%s"?>\n' % self.outputEncodingName)
 
         self._write(f, self.e)
