@@ -1,3 +1,4 @@
+from __future__ import unicode_literals, absolute_import, print_function, division
 #########################################################################
 #                                                                       #
 #                                                                       #
@@ -13,10 +14,13 @@
 import os, re
 from calibre.ebooks.rtf2xml import copy
 from calibre.ptempfile import better_mktemp
+from . import open_for_read, open_for_write
+
 
 class HeadingsToSections:
     """
     """
+
     def __init__(self,
             in_file,
             bug_handler,
@@ -37,6 +41,7 @@ class HeadingsToSections:
         self.__bug_handler = bug_handler
         self.__copy = copy
         self.__write_to = better_mktemp()
+
     def __initiate_values(self):
         """
         Required:
@@ -79,6 +84,7 @@ class HeadingsToSections:
         ]
         self.__section_num = [0]
         self.__id_regex = re.compile(r'\<list-id\>(\d+)')
+
     def __close_lists(self):
         """
         Required:
@@ -104,6 +110,7 @@ class HeadingsToSections:
                 num_levels_closed += 1
         self.__all_lists = self.__all_lists[num_levels_closed:]
         self.__all_lists.reverse()
+
     def __close_sections(self, current_level):
         self.__all_sections.reverse()
         num_levels_closed = 0
@@ -113,6 +120,7 @@ class HeadingsToSections:
                 num_levels_closed += 1
         self.__all_sections = self.__all_sections[num_levels_closed:]
         self.__all_sections.reverse()
+
     def __write_start_section(self, current_level, name):
         section_num = ''
         for the_num in self.__section_num:
@@ -129,9 +137,11 @@ class HeadingsToSections:
                 '<type>%s\n'
                 % (section_num, num_in_level, level, name)
                 )
+
     def __write_end_section(self):
         self.__write_obj.write('mi<mk<sect-close\n')
         self.__write_obj.write('mi<tg<close_____<section\n')
+
     def __default_func(self, line):
         """
         Required:
@@ -160,6 +170,7 @@ class HeadingsToSections:
         if self.__token_info == 'mi<mk<body-close':
             self.__state = 'after_body'
         self.__write_obj.write(line)
+
     def __handle_heading(self, name):
         num = self.__headings.index(name) + 1
         self.__close_sections(num)
@@ -171,10 +182,12 @@ class HeadingsToSections:
         else:
             self.__section_num[-1] += 1
         self.__write_start_section(num, name)
+
     def __in_table_func(self, line):
         if self.__token_info == 'mi<mk<table-end_':
             self.__state = 'default'
         self.__write_obj.write(line)
+
     def __in_list_func(self, line):
         if self.__token_info == 'mi<mk<list_close':
             self.__list_depth -= 1
@@ -183,8 +196,10 @@ class HeadingsToSections:
         if self.__list_depth == 0:
             self.__state = 'default'
         self.__write_obj.write(line)
+
     def __after_body_func(self, line):
         self.__write_obj.write(line)
+
     def make_sections(self):
         """
         Required:
@@ -194,8 +209,8 @@ class HeadingsToSections:
         Logic:
         """
         self.__initiate_values()
-        read_obj = open(self.__file, 'r')
-        self.__write_obj = open(self.__write_to, 'w')
+        read_obj = open_for_read(self.__file)
+        self.__write_obj = open_for_write(self.__write_to)
         line_to_read = 1
         while line_to_read:
             line_to_read = read_obj.readline()
