@@ -2,8 +2,6 @@
 # License: GPLv3 Copyright: 2008, Kovid Goyal <kovid at kovidgoyal.net>
 from __future__ import absolute_import, division, print_function, unicode_literals
 
-from functools import partial
-
 from PyQt5.Qt import (
     QBrush, QCheckBox, QCoreApplication, QDialog, QGridLayout, QHBoxLayout, QIcon,
     QKeySequence, QLabel, QListView, QModelIndex, QPalette, QPixmap, QPushButton,
@@ -14,10 +12,11 @@ from PyQt5.QtWebKitWidgets import QWebView
 
 from calibre import fit_image
 from calibre.gui2 import NO_URL_FORMATTING, gprefs
-from calibre.gui2.book_details import css, details_context_menu_event, render_html
+from calibre.gui2.book_details import css, details_context_menu_event, render_html, set_html
 from calibre.gui2.ui import get_gui
 from calibre.gui2.widgets import CoverView
 from calibre.gui2.widgets2 import Dialog
+from polyglot.builtins import unicode_type
 
 
 class Configure(Dialog):
@@ -40,14 +39,14 @@ class Configure(Dialog):
         h.addWidget(fdo)
         v = QVBoxLayout()
         self.mub = b = QToolButton(self)
-        b.clicked.connect(partial(move_field_up, fdo, self.model))
+        connect_lambda(b.clicked, self, lambda self: move_field_up(fdo, self.model))
         b.setIcon(QIcon(I('arrow-up.png')))
         b.setToolTip(_('Move the selected field up'))
         v.addWidget(b), v.addStretch(10)
         self.mud = b = QToolButton(self)
         b.setIcon(QIcon(I('arrow-down.png')))
         b.setToolTip(_('Move the selected field down'))
-        b.clicked.connect(partial(move_field_down, fdo, self.model))
+        connect_lambda(b.clicked, self, lambda self: move_field_down(fdo, self.model))
         v.addWidget(b)
         h.addLayout(v)
 
@@ -57,9 +56,19 @@ class Configure(Dialog):
 
         b = self.bb.addButton(_('Restore &defaults'), self.bb.ActionRole)
         b.clicked.connect(self.restore_defaults)
+        b = self.bb.addButton(_('Select &all'), self.bb.ActionRole)
+        b.clicked.connect(self.select_all)
+        b = self.bb.addButton(_('Select &none'), self.bb.ActionRole)
+        b.clicked.connect(self.select_none)
         self.l.addWidget(self.bb)
         self.setMinimumHeight(500)
 
+    def select_all(self):
+        self.model.toggle_all(True)
+
+    def select_none(self):
+        self.model.toggle_all(False)
+
     def restore_defaults(self):
         self.model.initialize(use_defaults=True)
 
@@ -146,9 +155,9 @@ class BookInfo(QDialog):
         self.ps = QShortcut(QKeySequence('Alt+Left'), self)
         self.ps.activated.connect(self.previous)
         self.next_button.setToolTip(_('Next [%s]')%
-                unicode(self.ns.key().toString(QKeySequence.NativeText)))
+                unicode_type(self.ns.key().toString(QKeySequence.NativeText)))
         self.previous_button.setToolTip(_('Previous [%s]')%
-                unicode(self.ps.key().toString(QKeySequence.NativeText)))
+                unicode_type(self.ps.key().toString(QKeySequence.NativeText)))
 
         geom = QCoreApplication.instance().desktop().availableGeometry(self)
         screen_height = geom.height() - 100
@@ -171,7 +180,7 @@ class BookInfo(QDialog):
                     self.refresh(self.current_row, mi=mi)
 
     def link_clicked(self, qurl):
-        link = unicode(qurl.toString(NO_URL_FORMATTING))
+        link = unicode_type(qurl.toString(NO_URL_FORMATTING))
         self.link_delegate(link)
 
     def done(self, r):
@@ -275,7 +284,7 @@ class BookInfo(QDialog):
         self.cover_pixmap.setDevicePixelRatio(dpr)
         self.resize_cover()
         html = render_html(mi, self.css, True, self, pref_name='popup_book_display_fields')
-        self.details.setHtml(html)
+        set_html(mi, html, self.details)
         self.marked = mi.marked
         self.cover.setBackgroundBrush(self.marked_brush if mi.marked else self.normal_brush)
         self.update_cover_tooltip()
