@@ -1,14 +1,15 @@
+from __future__ import print_function
 __license__   = 'GPL v3'
 __copyright__ = '2008, Kovid Goyal <kovid at kovidgoyal.net>'
-import sys, collections, operator, copy, re
+import sys, collections, operator, copy, re, numbers
 
 from PyQt5.Qt import (
     Qt, QRectF, QFont, QColor, QPixmap, QGraphicsPixmapItem, QGraphicsItem,
     QFontMetrics, QPen, QBrush, QGraphicsRectItem)
 
 from calibre.ebooks.lrf.fonts import LIBERATION_FONT_MAP
-from calibre.ebooks.BeautifulSoup import Tag
 from calibre.ebooks.hyphenate import hyphenate_word
+from polyglot.builtins import unicode_type, string_or_bytes
 
 WEIGHT_MAP = lambda wt : int((wt/10.)-1)
 NULL       = lambda a, b: a
@@ -162,8 +163,13 @@ class TextBlock(object):
         pass
 
     has_content = property(fget=lambda self: self.peek_index < len(self.lines)-1)
-    XML_ENTITIES = dict(zip(Tag.XML_SPECIAL_CHARS_TO_ENTITIES.values(), Tag.XML_SPECIAL_CHARS_TO_ENTITIES.keys()))
-    XML_ENTITIES["quot"] = '"'
+    XML_ENTITIES = {
+            "apos" : "'",
+            "quot" : '"',
+            "amp" : "&",
+            "lt" : "<",
+            "gt" : ">"
+    }
 
     def __init__(self, tb, font_loader, respect_max_y, text_width, logger,
                  opts, ruby_tags, link_activated):
@@ -182,7 +188,7 @@ class TextBlock(object):
         self.font_loader, self.logger, self.opts = font_loader, logger, opts
         self.in_link = False
         self.link_activated = link_activated
-        self.max_y = self.bs.blockheight if (respect_max_y or self.bs.blockrule.lower() in ('vert-fixed', 'block-fixed')) else sys.maxint
+        self.max_y = self.bs.blockheight if (respect_max_y or self.bs.blockrule.lower() in ('vert-fixed', 'block-fixed')) else sys.maxsize
         self.height = 0
         self.peek_index = -1
 
@@ -221,7 +227,7 @@ class TextBlock(object):
         open_containers = collections.deque()
         self.in_para = False
         for i in tb.content:
-            if isinstance(i, basestring):
+            if isinstance(i, string_or_bytes):
                 self.process_text(i)
             elif i is None:
                 if len(open_containers) > 0:
@@ -309,6 +315,8 @@ class TextBlock(object):
             raw = raw[pos:]
             if line_filled:
                 self.end_line()
+            if not pos:
+                break
 
     def __iter__(self):
         for line in self.lines:
@@ -367,7 +375,7 @@ class Line(QGraphicsItem):
             self.children = self.childItems
 
     def start_link(self, refobj, slot):
-        self.current_link = [self.current_width, sys.maxint, refobj, slot]
+        self.current_link = [self.current_width, sys.maxsize, refobj, slot]
 
     def end_link(self):
         if self.current_link is not None:
@@ -449,7 +457,7 @@ class Line(QGraphicsItem):
         if self.length_in_space > 0:
             frac = 1 + float(delta)/self.length_in_space
             for i in range(len(self.tokens)):
-                if isinstance(self.tokens[i], (int, float)):
+                if isinstance(self.tokens[i], numbers.Number):
                     self.tokens[i] *= frac
             self.current_width = self.line_length
 
@@ -493,7 +501,7 @@ class Line(QGraphicsItem):
         painter.restore()
         painter.save()
         for tok in self.tokens:
-            if isinstance(tok, (int, float)):
+            if isinstance(tok, numbers.Number):
                 x += tok
             elif isinstance(tok, Word):
                 painter.setFont(tok.font)
@@ -530,14 +538,14 @@ class Line(QGraphicsItem):
         matches = []
         try:
             while True:
-                word = words.next()
+                word = next(words)
                 word.highlight = False
-                if tokens[0] in unicode(word.string).lower():
+                if tokens[0] in unicode_type(word.string).lower():
                     matches.append(word)
                     for c in range(1, len(tokens)):
-                        word = words.next()
-                        print tokens[c], word.string
-                        if tokens[c] not in unicode(word.string):
+                        word = next(words)
+                        print(tokens[c], word.string)
+                        if tokens[c] not in unicode_type(word.string):
                             return None
                         matches.append(word)
                     for w in matches:
@@ -557,14 +565,14 @@ class Line(QGraphicsItem):
     def __unicode__(self):
         s = u''
         for tok in self.tokens:
-            if isinstance(tok, (int, float)):
+            if isinstance(tok, numbers.Number):
                 s += ' '
             elif isinstance(tok, Word):
-                s += unicode(tok.string)
+                s += unicode_type(tok.string)
         return s
 
     def __str__(self):
-        return unicode(self).encode('utf-8')
+        return unicode_type(self).encode('utf-8')
 
 
 class Word(object):
@@ -580,5 +588,6 @@ class Word(object):
 def main(args=sys.argv):
     return 0
 
+
 if __name__ == '__main__':
     sys.exit(main())
