@@ -1,13 +1,17 @@
 # -*- coding: utf-8 -*-
+from __future__ import absolute_import, division, print_function, unicode_literals
 
-from __future__ import (unicode_literals, division, absolute_import, print_function)
-store_version = 6  # Needed for dynamic plugin loading
+store_version = 9  # Needed for dynamic plugin loading
 
 __license__ = 'GPL 3'
-__copyright__ = '2012-2016, Tomasz Długosz <tomek3d@gmail.com>'
+__copyright__ = '2012-2017, Tomasz Długosz <tomek3d@gmail.com>'
 __docformat__ = 'restructuredtext en'
 
-import urllib
+try:
+    from urllib.parse import quote
+except ImportError:
+    from urllib import quote
+from base64 import b64encode
 from contextlib import closing
 
 from lxml import html
@@ -22,16 +26,31 @@ from calibre.gui2.store.search_result import SearchResult
 from calibre.gui2.store.web_store_dialog import WebStoreDialog
 
 
+def as_base64(data):
+    if not isinstance(data, bytes):
+        data = data.encode('utf-8')
+    ans = b64encode(data)
+    if isinstance(ans, bytes):
+        ans = ans.decode('ascii')
+    return ans
+
+
 class PublioStore(BasicStoreConfig, StorePlugin):
 
     def open(self, parent=None, detail_item=None, external=False):
-        google_analytics = '?utm_source=tdcalibre&utm_medium=calibre'
-        url = 'http://www.publio.pl/' + google_analytics
+        aff_root = 'https://www.a4b-tracking.com/pl/stat-click-text-link/29/58/'
+        url = 'http://www.publio.pl/'
+
+        aff_url = aff_root + as_base64(url)
+
+        detail_url = None
+        if detail_item:
+            detail_url = aff_root + as_base64(detail_item)
 
         if external or self.config.get('open_external', False):
-            open_url(QUrl(url_slash_cleaner((detail_item + google_analytics) if detail_item else url)))
+            open_url(QUrl(url_slash_cleaner(detail_url if detail_url else aff_url)))
         else:
-            d = WebStoreDialog(self.gui, url, parent, detail_item if detail_item else url)
+            d = WebStoreDialog(self.gui, url, parent, detail_url if detail_url else aff_url)
             d.setWindowTitle(self.name)
             d.set_tags(self.config.get('tags', ''))
             d.exec_()
@@ -43,9 +62,9 @@ class PublioStore(BasicStoreConfig, StorePlugin):
         counter = max_results
         page = 1
         while counter:
-            with closing(br.open('http://www.publio.pl/szukaj,strona' + str(page) + '.html?q=' + urllib.quote(query) + '&sections=EMAGAZINE&sections=MINIBOOK&sections=EBOOK', timeout=timeout)) as f:  # noqa
+            with closing(br.open('http://www.publio.pl/e-booki,strona' + str(page) + '.html?q=' + quote(query), timeout=timeout)) as f:  # noqa
                 doc = html.fromstring(f.read())
-                for data in doc.xpath('//div[@class="product-tile"]'):
+                for data in doc.xpath('//div[@class="products-list"]//div[@class="product-tile"]'):
                     if counter <= 0:
                         break
 
@@ -54,10 +73,10 @@ class PublioStore(BasicStoreConfig, StorePlugin):
                         continue
 
                     cover_url = ''.join(data.xpath('.//img[@class="product-tile-cover-photo"]/@src'))
-                    title = ''.join(data.xpath('.//h3[@class="product-tile-title"]/a/span[1]/text()'))
+                    title = ''.join(data.xpath('.//span[@class="product-tile-title-long"]/text()'))
                     author = ', '.join(data.xpath('.//span[@class="product-tile-author"]/a/text()'))
                     price = ''.join(data.xpath('.//div[@class="product-tile-price-wrapper "]/a/ins/text()'))
-                    # formats = ', '.join([x.strip() for x in data.xpath('.//div[@class="formats"]/a/text()')])
+                    formats = ''.join(data.xpath('.//a[@class="product-tile-cover"]/img/@alt')).split(' - ebook ')[1]
 
                     counter -= 1
 
@@ -67,8 +86,7 @@ class PublioStore(BasicStoreConfig, StorePlugin):
                     s.author = author
                     s.price = price
                     s.detail_item = 'http://www.publio.pl' + id.strip()
-                    # s.drm = SearchResult.DRM_LOCKED if 'DRM' in formats else SearchResult.DRM_UNLOCKED
-                    # s.formats = formats.replace(' DRM','').strip()
+                    s.formats = formats.upper().strip()
 
                     yield s
                 if not doc.xpath('boolean(//a[@class="next"])'):
